@@ -48,14 +48,23 @@
 #define NAUTILUS_COMMAND	 BIN_NAUTILUS" -n --no-desktop %m"
 
 static struct gvm_configuration config;
+static DBusConnection *dbus_connection;
 static LibHalContext *hal_ctx;
 
-/** Table of UDI's for volumes being mounted by g-v-m that we need to apply policy to */
+struct _MountPolicy {
+	char *udi;
+	int apply;
+};
+
+/* Table of UDI's for volumes being mounted by g-v-m that we need to apply policy to */
 static GHashTable *mount_table = NULL;
 
-/** List of UDI's of all volumes mounted during the lifetime of the program */
+/* List of UDI's of all volumes mounted during the lifetime of the program */
 static GSList *mounted_volumes = NULL;
 
+/* Table of active dialogs indexed by the udi of the device in question */
+static GHashTable *dialogs = NULL;
+
 
 typedef enum {
 	TYPE_BOOL,
@@ -71,21 +80,22 @@ enum {
 	AUTOIPOD_COMMAND,
 	AUTOMOUNT_DRIVES,
 	AUTOMOUNT_MEDIA,
-	AUTOPALMSYNC,
-	AUTOPALMSYNC_COMMAND,
 	AUTOPHOTO,
 	AUTOPHOTO_COMMAND,
+	AUTOPILOT,
+	AUTOPILOT_COMMAND,
 	AUTOPLAY_CDA,
 	AUTOPLAY_CDA_COMMAND,
 	AUTOPLAY_DVD,
 	AUTOPLAY_DVD_COMMAND,
 	AUTOPLAY_VCD,
 	AUTOPLAY_VCD_COMMAND,
+	AUTOPOCKETPC,
+	AUTOPOCKETPC_COMMAND,
 	AUTOPRINTER,
 	AUTOPRINTER_COMMAND,
 	AUTORUN,
 	AUTORUN_PATH,
-	EJECT_COMMAND,
 };
 
 static struct {
@@ -101,21 +111,22 @@ static struct {
 	{ GCONF_ROOT "autoipod_command",          TYPE_STRING, &config.autoipod_command          },
 	{ GCONF_ROOT "automount_drives",          TYPE_BOOL,   &config.automount_drives          },
 	{ GCONF_ROOT "automount_media",           TYPE_BOOL,   &config.automount_media           },
-	{ GCONF_ROOT "autopalmsync",              TYPE_BOOL,   &config.autopalmsync              },
-	{ GCONF_ROOT "autopalmsync_command",      TYPE_STRING, &config.autopalmsync_command      },
 	{ GCONF_ROOT "autophoto",                 TYPE_BOOL,   &config.autophoto                 },
 	{ GCONF_ROOT "autophoto_command",         TYPE_STRING, &config.autophoto_command         },
+	{ GCONF_ROOT "autopalmsync",              TYPE_BOOL,   &config.autopilot                 },
+	{ GCONF_ROOT "autopalmsync_command",      TYPE_STRING, &config.autopilot_command         },
 	{ GCONF_ROOT "autoplay_cda",              TYPE_BOOL,   &config.autoplay_cda              },
 	{ GCONF_ROOT "autoplay_cda_command",      TYPE_STRING, &config.autoplay_cda_command      },
 	{ GCONF_ROOT "autoplay_dvd",              TYPE_BOOL,   &config.autoplay_dvd              },
 	{ GCONF_ROOT "autoplay_dvd_command",      TYPE_STRING, &config.autoplay_dvd_command      },
 	{ GCONF_ROOT "autoplay_vcd",              TYPE_BOOL,   &config.autoplay_vcd              },
 	{ GCONF_ROOT "autoplay_vcd_command",      TYPE_STRING, &config.autoplay_vcd_command      },
+	{ GCONF_ROOT "autopocketpc",              TYPE_BOOL,   &config.autopocketpc              },
+	{ GCONF_ROOT "autopocketpc_command",      TYPE_STRING, &config.autopocketpc_command      },
 	{ GCONF_ROOT "autoprinter",               TYPE_BOOL,   &config.autoprinter               },
 	{ GCONF_ROOT "autoprinter_command",       TYPE_STRING, &config.autoprinter_command       },
 	{ GCONF_ROOT "autorun",                   TYPE_BOOL,   &config.autorun                   },
 	{ GCONF_ROOT "autorun_path",              TYPE_STRING, &config.autorun_path              },
-	{ GCONF_ROOT "eject_command",             TYPE_STRING, &config.eject_command             },
 };
 
 static GHashTable *gvm_settings_hash = NULL;
@@ -145,12 +156,12 @@ static struct _GvmPromptButton GVM_BUTTONS_AUTORUN[] = {
 
 static struct _GvmPromptButton GVM_BUTTONS_CAMERA[] = {
 	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
-	{ N_("_Open Folder"), NULL, GVM_RESPONSE_BROWSE },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
 };
 
-static struct _GvmPromptButton GVM_BUTTONS_PTP_CAMERA[] = {
+static struct _GvmPromptButton GVM_BUTTONS_STORAGE_CAMERA[] = {
 	{ N_("_Ignore"), NULL, GTK_RESPONSE_CANCEL },
+	{ N_("_Open Folder"), NULL, GVM_RESPONSE_BROWSE },
 	{ N_("Import _Photos"), NULL, GVM_RESPONSE_IMPORT_PHOTOS },
 };
 
@@ -180,7 +191,7 @@ static struct _GvmPromptButton GVM_BUTTONS_BURN_DVD[] = {
 typedef enum {
 	GVM_PROMPT_AUTORUN,
 	GVM_PROMPT_IMPORT_CAMERA,
-	GVM_PROMPT_IMPORT_PTP_CAMERA,
+	GVM_PROMPT_IMPORT_STORAGE_CAMERA,
 	GVM_PROMPT_IMPORT_PHOTOS,
 	GVM_PROMPT_IPOD_PHOTO,
 	GVM_PROMPT_CDA_EXTRA,
@@ -238,12 +249,12 @@ static struct {
 	  N_("There are photos on the camera. Would you like to add these pictures to your album?"), FALSE, FALSE,
 	  GCONF_ROOT "prompts/camera_import_photos", N_("_Always perform this action") },
 	{ 0, "applets-screenshooter", NULL,
-	  GVM_BUTTONS_PTP_CAMERA, G_N_ELEMENTS (GVM_BUTTONS_PTP_CAMERA),
+	  GVM_BUTTONS_STORAGE_CAMERA, G_N_ELEMENTS (GVM_BUTTONS_STORAGE_CAMERA),
 	  GVM_RESPONSE_IMPORT_PHOTOS,
 	  N_("Camera Import"),
 	  N_("A camera has been detected."),
 	  N_("There are photos on the camera. Would you like to add these pictures to your album?"), FALSE, FALSE,
-	  GCONF_ROOT "prompts/ptp_camera_import_photos", N_("_Always perform this action") },
+	  GCONF_ROOT "prompts/storage_camera_import_photos", N_("_Always perform this action") },
 	{ 0, "applets-screenshooter", NULL,
 	  GVM_BUTTONS_CAMERA, G_N_ELEMENTS (GVM_BUTTONS_CAMERA),
 	  GVM_RESPONSE_IMPORT_PHOTOS,
@@ -269,14 +280,14 @@ static struct {
 	  GVM_BUTTONS_BURN_CD, G_N_ELEMENTS (GVM_BUTTONS_BURN_CD),
 	  GVM_RESPONSE_BURN_DATA_CD,
 	  N_("Choose Disc Type"),
-	  N_("You have inserted an empty disc."),
+	  N_("You have inserted a blank disc."),
 	  N_("What would you like to do?"), FALSE, FALSE,
 	  NULL, NULL },
 	{ 0, "gnome-dev-disc-dvdr", NULL,
 	  GVM_BUTTONS_BURN_DVD, G_N_ELEMENTS (GVM_BUTTONS_BURN_DVD),
 	  GVM_RESPONSE_BURN_DATA_CD,
 	  N_("Choose Disc Type"),
-	  N_("You have inserted an empty disc."),
+	  N_("You have inserted a blank disc."),
 	  N_("What would you like to do?"), FALSE, FALSE,
 	  NULL, NULL },
 };
@@ -331,6 +342,7 @@ prompt_response_cb (GtkWidget *dialog, int response, GvmPromptCtx *ctx)
 			g_object_unref (gconf);
 		}
 		
+		g_hash_table_remove (dialogs, ctx->udi);
 		ctx->callback (ctx, response);
 		gtk_widget_destroy (dialog);
 		gvm_prompt_ctx_free (ctx);
@@ -382,9 +394,10 @@ gvm_prompt (GvmPromptCtx *ctx, int argc, char **argv)
 {
 	GtkWidget *dialog, *hbox, *vbox, *image, *label, *check = NULL;
 	GvmPrompt prompt = ctx->prompt;
-	GnomeIconTheme *theme;
+	GtkIconTheme *theme;
 	GConfClient *gconf;
 	GError *err = NULL;
+	GdkPixbuf *pixbuf;
 	const char *text;
 	int response, i;
 	char *buf;
@@ -440,12 +453,12 @@ gvm_prompt (GvmPromptCtx *ctx, int argc, char **argv)
 	gtk_container_set_border_width ((GtkContainer *) hbox, 12);
 	
 	/* set the icon */
-	theme = gnome_icon_theme_new ();
+	theme = gtk_icon_theme_get_default ();
 	image = gtk_image_new ();
 	
-	if ((buf = gnome_icon_theme_lookup_icon (theme, gvm_prompts[prompt].icon, 48, NULL, NULL))) {
-		gtk_image_set_from_file ((GtkImage *) image, buf);
-		g_free (buf);
+	if ((pixbuf = gtk_icon_theme_load_icon (theme, gvm_prompts[prompt].icon, 48, 0, NULL))) {
+		gtk_image_set_from_pixbuf ((GtkImage *) image, pixbuf);
+		g_object_unref (pixbuf);
 	} else {
 	}
 	
@@ -505,6 +518,8 @@ gvm_prompt (GvmPromptCtx *ctx, int argc, char **argv)
 	
 	g_object_unref (gconf);
 	
+	g_hash_table_insert (dialogs, ctx->udi, dialog);
+	
 	g_signal_connect (dialog, "response", G_CALLBACK (prompt_response_cb), ctx);
 	
 	gtk_widget_show (dialog);
@@ -540,7 +555,7 @@ to_be_or_not_to_be (void)
 static void
 gvm_load_config (void)
 {
-	unsigned int i;
+	size_t i;
 	
 	gvm_settings_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	
@@ -759,19 +774,19 @@ gvm_check_vcd (const char *udi, const char *device, const char *mount_point)
 	
 	is_vcd = gvm_check_dir (mount_point, "vcd");
 	
-	if (is_vcd && config.autoplay_dvd)
+	if (is_vcd && config.autoplay_vcd)
 		gvm_run_command (config.autoplay_vcd_command, udi, device, mount_point);
 	
 	return is_vcd;
 }
 
 /*
- * gvm_udi_is_camera - checks if the udi is a camera device
+ * gvm_udi_is_storage_camera - checks if the udi is a mass-storage camera device
  *
  * Returns TRUE if the device is a camera or FALSE otherwise.
  */
 static gboolean
-gvm_udi_is_camera (const char *udi)
+gvm_udi_is_storage_camera (const char *udi)
 {
 	char *storage_device = NULL, *physical_device = NULL;
 	gboolean is_camera = FALSE;
@@ -804,15 +819,16 @@ gvm_udi_is_camera (const char *udi)
 }
 
 /*
- * gvm_udi_is_ptp_camera - checks if the udi is a PTP camera device
+ * gvm_udi_is_camera - checks if the udi is a supported non-mass-storage camera device
  *
- * Returns TRUE if the device is a PTP camera with libgphoto2 support or FALSE otherwise.
+ * Returns TRUE if the device is a camera with libgphoto2 support or FALSE otherwise.
  */
 static gboolean
-gvm_udi_is_ptp_camera (const char *udi)
+gvm_udi_is_camera (const char *udi)
 {
-	gboolean is_ptp_camera = FALSE;
+	gboolean is_camera = FALSE;
 	char *access_method;
+	char *driver;
 	
 #if 0
 	/* Note: this query is not necessary atm because our only caller already has this info */
@@ -823,14 +839,40 @@ gvm_udi_is_ptp_camera (const char *udi)
 	if (!(access_method = libhal_device_get_property_string (hal_ctx, udi, "camera.access_method", NULL)))
 		return FALSE;
 	
-	if (!strcmp (access_method, "ptp")) {
-		dbg ("PTP Camera detected: %s\n", udi);
-		is_ptp_camera = TRUE;
+	if (!strcmp (access_method, "storage")) {
+		/* we only want to match non-storage cameras */
+		is_camera = FALSE;
+		goto done;
+	} else if (!strcmp (access_method, "ptp")) {
+		/* ptp cameras are supported by libgphoto2 always */
+		is_camera = TRUE;
+		goto done;
+	} else if (!strcmp (access_method, "libgphoto2")) {
+		/* user has an old install of the libgphoto2 fdi files */
+		warn ("Please update your libgphoto2 package as the fdi files are outdated!\n");
+		if ((driver = libhal_device_get_property_string (hal_ctx, udi, "info.linux.driver", NULL))) {
+			/* the old fdi files marked everything as access_method="libgphoto2" so check that
+			 * this is a non-mass-storage camera */
+			if (!strcmp (driver, "usb-storage"))
+				is_camera = FALSE;
+		} else {
+			is_camera = TRUE;
+		}
+		
+		libhal_free_string (driver);
+		
+		goto done;
 	}
 	
+	dbg ("Non Mass-Storage Camera detected: %s\n", udi);
+	
+	is_camera = libhal_device_get_property_bool (hal_ctx, udi, "camera.libgphoto2.support", NULL);
+	
+ done:
+	
 	libhal_free_string (access_method);
 	
-	return is_ptp_camera;
+	return is_camera;
 }
 
 /*
@@ -920,15 +962,15 @@ gvm_run_ipod (const char *udi, const char *device, const char *mount_point)
 
 
 /*
- * gvm_run_palmsync - sync the pda palm pilot
+ * gvm_run_pilot - sync the pda palm pilot
  */
 static void
-gvm_run_palmsync (const char *udi)
+gvm_run_pilot (const char *udi)
 {
 	DBusError error;
 	char *device;
 	
-	if (config.autopalmsync_command == NULL)
+	if (config.autopilot_command == NULL)
 		return;
 	
 	dbus_error_init (&error);
@@ -940,7 +982,32 @@ gvm_run_palmsync (const char *udi)
 		return;
 	}
 	
-	gvm_run_command (config.autopalmsync_command, udi, device, NULL);
+	gvm_run_command (config.autopilot_command, udi, device, NULL);
+	libhal_free_string (device);
+}
+
+/*
+ * gvm_run_pilot - sync the pda palm pilot
+ */
+static void
+gvm_run_pocketpc (const char *udi)
+{
+	DBusError error;
+	char *device;
+	
+	if (config.autopocketpc_command == NULL)
+		return;
+	
+	dbus_error_init (&error);
+	/* pda.palm.hotsync_interface should be the same as serial.device, but just in case... */
+	if (!(device = libhal_device_get_property_string (hal_ctx, udi, "pda.pocketpc.hotsync_interface", &error))) {
+		warn ("cannot get pda.pocketpc.hotsync_interface property: %s", error.message);
+		if (dbus_error_is_set (&error))
+			dbus_error_free (&error);
+		return;
+	}
+	
+	gvm_run_command (config.autopocketpc_command, udi, device, NULL);
 	libhal_free_string (device);
 }
 
@@ -1006,7 +1073,7 @@ autorun_cb (GvmPromptCtx *ctx, int action)
 static void
 gvm_autorun (const char *udi, const char *device, const char *mount_point)
 {
-	gboolean autobrowse = FALSE;
+	gboolean autobrowse = TRUE;
 	GvmPromptCtx *ctx;
 	
 	if (gvm_check_dvd (udi, device, mount_point))
@@ -1075,7 +1142,7 @@ gvm_udi_is_subfs_mount (const char *udi)
 }
 
 static void
-import_camera_cb (GvmPromptCtx *ctx, int action)
+import_storage_camera_cb (GvmPromptCtx *ctx, int action)
 {
 	switch (action) {
 	case GVM_RESPONSE_IMPORT_PHOTOS:
@@ -1134,8 +1201,8 @@ gvm_device_mounted (const char *udi)
 	}
 	
 	/* this is where the magic happens */
-	if (config.autophoto && gvm_udi_is_camera (udi)) {
-		ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_CAMERA, import_camera_cb, udi, device, mount_point, NULL);
+	if (config.autophoto && gvm_udi_is_storage_camera (udi)) {
+		ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_STORAGE_CAMERA, import_storage_camera_cb, udi, device, mount_point, NULL);
 		gvm_prompt (ctx, 0, NULL);
 	} else if (config.autoipod && gvm_udi_is_ipod (udi)) {
 		char *ipod_control;
@@ -1168,33 +1235,101 @@ gvm_device_mounted (const char *udi)
  * @return TRUE iff the mount was succesful
  */
 static gboolean
-gvm_device_mount (const char *udi, const char *device, const char *mount_point)
+gvm_device_mount (const char *udi, int apply_policy)
 {
-	char *key;
-	
-	key = g_strdup (udi);
-	g_hash_table_insert (mount_table, key, key);
+	char *mount_point, *fstype, *options[2];
+	struct _MountPolicy *policy;
+	DBusMessage *dmesg, *reply;
+	dbus_uint64_t size;
+	int noptions = 0;
+	DBusError error;
 	
 	dbg ("mounting %s...\n", udi);
-	if (gvm_run_command (MOUNT_COMMAND, udi, device, mount_point))
-		return TRUE;
 	
-	dbg ("mount failed: %s\n", udi);
-	g_hash_table_remove (mount_table, key);
-	g_free (key);
+	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+						    "org.freedesktop.Hal.Device.Volume",
+						    "Mount"))) {
+		dbg ("mount failed for %s: could not create dbus message\n", udi);
+		return FALSE;
+	}
 	
-	return FALSE;
+	size = libhal_device_get_property_uint64 (hal_ctx, udi, "volume.size", NULL);
+	
+	fstype = "";
+	mount_point = "";
+	
+	if (size <= (256 * 1024 * 1024))
+		options[noptions++] = "sync";
+	options[noptions] = NULL;
+	
+	if (!dbus_message_append_args (dmesg, DBUS_TYPE_STRING, &mount_point, DBUS_TYPE_STRING, &fstype,
+				       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options, noptions,
+				       DBUS_TYPE_INVALID)) {
+		dbg ("mount failed for %s: could not append args to dbus message\n", udi);
+		dbus_message_unref (dmesg);
+		return FALSE;
+	}
+	
+	policy = g_new (struct _MountPolicy, 1);
+	policy->udi = g_strdup (udi);
+	policy->apply = apply_policy;
+	
+	g_hash_table_insert (mount_table, policy->udi, policy);
+	
+	dbus_error_init (&error);
+	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+		dbg ("mount failed for %s: %s\n", udi, error.message);
+		g_hash_table_remove (mount_table, policy->udi);
+		dbus_message_unref (dmesg);
+		dbus_error_free (&error);
+		g_free (policy->udi);
+		g_free (policy);
+		return FALSE;
+	}
+	
+	dbg ("mount queued for %s\n", udi);
+	
+	dbus_message_unref (dmesg);
+	dbus_message_unref (reply);
+	
+	return TRUE;
 }
 
+
 /*
  * gvm_device_unmount - unmount the given device.
  *
  * @return TRUE iff the unmount was succesful
  */
 static gboolean
-gvm_device_unmount (const char *udi, const char *device, const char *mount_point)
+gvm_device_unmount (const char *udi)
 {
-	return gvm_run_command (UNMOUNT_COMMAND, udi, device, mount_point);
+	DBusMessage *dmesg, *reply;
+	DBusError error;
+	
+	dbg ("unmounting %s...\n", udi);
+	
+	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+						    "org.freedesktop.Hal.Device.Volume",
+						    "Unmount"))) {
+		dbg ("unmount failed for %s: could not create dbus message\n", udi);
+		return FALSE;
+	}
+	
+	dbus_error_init (&error);
+	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+		dbg ("unmount failed for %s: %s\n", udi, error.message);
+		dbus_message_unref (dmesg);
+		dbus_error_free (&error);
+		return FALSE;
+	}
+	
+	dbg ("unmount queued for %s\n", udi);
+	
+	dbus_message_unref (dmesg);
+	dbus_message_unref (reply);
+	
+	return TRUE;
 }
 
 /*
@@ -1214,7 +1349,7 @@ cda_extra_cb (GvmPromptCtx *ctx, int action)
 	switch (action) {
 	case GVM_RESPONSE_BROWSE:
 		if (!gvm_udi_is_subfs_mount (ctx->udi))
-			gvm_device_mount (ctx->udi, ctx->device, NULL);
+			gvm_device_mount (ctx->udi, TRUE);
 		break;
 	case GVM_RESPONSE_PLAY:
 		gvm_run_cdplayer (ctx->udi, ctx->device, NULL);
@@ -1249,7 +1384,7 @@ gvm_ask_mixed (const char *udi)
 		gvm_prompt (ctx, 0, NULL);
 	} else if (config.automount_media) {
 		if (!gvm_udi_is_subfs_mount (udi))
-			gvm_device_mount (udi, device, NULL);
+			gvm_device_mount (udi, TRUE);
 	} else if (config.autoplay_cda) {
 		gvm_run_cdplayer (udi, device, NULL);
 	}
@@ -1257,41 +1392,57 @@ gvm_ask_mixed (const char *udi)
 	libhal_free_string (device);
 }
 
-enum {
+
+typedef enum {
 	WRITER_TYPE_NONE,
 	WRITER_TYPE_CDR,
 	WRITER_TYPE_DVD
+} writer_t;
+
+static struct {
+	const char *disc;
+	const char *drive;
+	writer_t type;
+} burners[] = {
+	{ "cd_r",        "storage.cdrom.cdr",       WRITER_TYPE_CDR },
+	{ "cd_rw",       "storage.cdrom.cdrw",      WRITER_TYPE_CDR },
+	{ "dvd_r",       "storage.cdrom.dvdr",      WRITER_TYPE_DVD },
+	{ "dvd_rw",      "storage.cdrom.dvdrw",     WRITER_TYPE_DVD },
+	{ "dvd_ram",     "storage.cdrom.dvdram",    WRITER_TYPE_DVD },
+	{ "dvd_plus_r",  "storage.cdrom.dvdplusr",  WRITER_TYPE_DVD },
+	{ "dvd_plus_rw", "storage.cdrom.dvdplusrw", WRITER_TYPE_DVD },
 };
 
 /*
- * gvm_device_is_writer - is this device capable of writing CDs/DVDs?
+ * gvm_cdrom_media_is_writable - returns the type of media that can be written
  */
-static int
-gvm_device_is_writer (const char *udi)
+static writer_t
+gvm_cdrom_media_is_writable (const char *udi)
 {
-	static const char *writers[] = {
-		"storage.cdrom.cdr",
-		"storage.cdrom.cdrw",
-		"storage.cdrom.dvdr",
-		"storage.cdrom.dvdram",
-		"storage.cdrom.dvdplusr",
-		"storage.cdrom.dvdplusrw"
-	};
+	writer_t retval = WRITER_TYPE_NONE;
+	char *drive = NULL;
+	char *disc = NULL;
 	size_t i;
 	
-	for (i = 0; i < G_N_ELEMENTS (writers); i++) {
-		if (libhal_device_get_property_bool (hal_ctx, udi, writers[i], NULL)) {
-			const char *type;
+	if (!(disc = libhal_device_get_property_string (hal_ctx, udi, "volume.disc.type", NULL)))
+		return FALSE;
+	
+	for (i = 0; i < G_N_ELEMENTS (burners); i++) {
+		if (!strcmp (burners[i].disc, disc)) {
+			if (!(drive = libhal_device_get_property_string (hal_ctx, udi, "info.parent", NULL)))
+				break;
 			
-			type = strrchr (writers[i], '.') + 1;
-			if (!strncmp (type, "dvd", 3))
-				return WRITER_TYPE_DVD;
-			else
-				return WRITER_TYPE_CDR;
+			if (libhal_device_get_property_bool (hal_ctx, drive, burners[i].drive, NULL))
+				retval = burners[i].type;
+			
+			break;
 		}
 	}
 	
-	return WRITER_TYPE_NONE;
+	libhal_free_string (drive);
+	libhal_free_string (disc);
+	
+	return retval;
 }
 
 static void
@@ -1343,11 +1494,10 @@ gvm_run_cdburner (const char *udi, int type, const char *device, const char *mou
 static void
 gvm_cdrom_policy (const char *udi)
 {
-	char *device = NULL;
-	char *drive_udi = NULL;
 	dbus_bool_t has_audio;
 	dbus_bool_t has_data;
 	dbus_bool_t is_blank;
+	char *device = NULL;
 	DBusError error;
 	int type;
 	
@@ -1355,13 +1505,11 @@ gvm_cdrom_policy (const char *udi)
 	has_audio = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_audio", NULL);
 	has_data = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.has_data", NULL);
 	is_blank = libhal_device_get_property_bool (hal_ctx, udi, "volume.disc.is_blank", NULL);
-	drive_udi = libhal_device_get_property_string (hal_ctx, udi, "info.parent", NULL);
-	device = libhal_device_get_property_string (hal_ctx, udi, "block.device", &error);
 	
-	if (!device) {
+	if (!(device = libhal_device_get_property_string (hal_ctx, udi, "block.device", &error))) {
 		warn ("cannot get block.device: %s", error.message);
 		dbus_error_free (&error);
-		goto out;
+		return;
 	}
 	
 	if (has_audio && !has_data) {
@@ -1371,18 +1519,15 @@ gvm_cdrom_policy (const char *udi)
 		gvm_ask_mixed (udi);
 	} else if (has_data) {
 		if (config.automount_media && !gvm_udi_is_subfs_mount (udi))
-			gvm_device_mount (udi, device, NULL);
+			gvm_device_mount (udi, TRUE);
 	} else if (is_blank) {
-		if ((type = gvm_device_is_writer (drive_udi)))
+		if ((type = gvm_cdrom_media_is_writable (udi)))
 			gvm_run_cdburner (udi, type, device, NULL);
 	}
 	
 	/** @todo enforce policy for all the new disc types now supported */
 	
- out:
-	
 	libhal_free_string (device);
-	libhal_free_string (drive_udi);
 }
 
 /*
@@ -1466,7 +1611,7 @@ gvm_automount_enabled (const char *udi)
 }
 
 static void
-import_ptp_camera_cb (GvmPromptCtx *ctx, int action)
+import_camera_cb (GvmPromptCtx *ctx, int action)
 {
 	if (action == GVM_RESPONSE_IMPORT_PHOTOS)
 		gvm_run_camera (ctx->udi, NULL, NULL);
@@ -1532,14 +1677,14 @@ block_device_added (const char *udi, const char *capability __attribute__((__unu
 	if (config.automount_drives && mountable) {
 		if (!gvm_udi_is_subfs_mount (udi)) {
 			if (gvm_automount_enabled (udi)) {
-				gvm_device_mount (udi, device, NULL);
+				gvm_device_mount (udi, TRUE);
 			} else {
 				dbg ("storage.automount_enabled_hint set to false on %s, not mounting\n", udi);
 			}
 		}
 	}
 	
-out:
+ out:
 	
 	if (dbus_error_is_set (&error))
 		dbus_error_free (&error);
@@ -1556,13 +1701,13 @@ camera_device_added (const char *udi, const char *capability __attribute__((__un
 {
 	GvmPromptCtx *ctx;
 	
-	if (!gvm_udi_is_ptp_camera (udi))
+	if (!gvm_udi_is_camera (udi))
 		return FALSE;
 	
 	if (!(config.autophoto && config.autophoto_command))
 		return TRUE;
 	
-	ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_PTP_CAMERA, import_ptp_camera_cb, udi, NULL, NULL, NULL);
+	ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_CAMERA, import_camera_cb, udi, NULL, NULL, NULL);
 	gvm_prompt (ctx, 0, NULL);
 	
 	return TRUE;
@@ -1593,12 +1738,12 @@ input_device_added (const char *udi, const char *capability)
 		if (!strcmp (inputs[i].capability, capability)) {
 			autoexec = *inputs[i].autoexec;
 			command = *inputs[i].command;
-			handled = TRUE;
 			break;
 		}
 	}
 	
 	if (autoexec && command) {
+		handled = TRUE;
 		dbus_error_init (&error);
 		if ((device = libhal_device_get_property_string (hal_ctx, udi, "input.device", &error))) {
 			gvm_run_command (command, udi, device, NULL);
@@ -1622,8 +1767,11 @@ pda_device_added (const char *udi, const char *capability __attribute__((__unuse
 	
 	if ((platform = libhal_device_get_property_string (hal_ctx, udi, "pda.platform", &error))) {
 		if (!strcmp (platform, "palm")) {
-			if (config.autopalmsync)
-				gvm_run_palmsync (udi);
+			if (config.autopilot)
+				gvm_run_pilot (udi);
+		} else if (!strcmp (platform, "pocketpc")) {
+			if (config.autopocketpc)
+				gvm_run_pocketpc (udi);
 		}
 		
 		libhal_free_string (platform);
@@ -1644,6 +1792,7 @@ printer_device_added (const char *udi, const char *capability __attribute__((__u
 	return TRUE;
 }
 
+/* Note: this list must be sorted alphabetically for the algorithm in hal_device_added to work */
 static struct {
 	const char *capability;
 	DeviceAddedHandler handler;
@@ -1669,7 +1818,6 @@ hal_device_added (LibHalContext *ctx __attribute__((__unused__)),
 {
 	char **capabilities;
 	size_t i, j, n;
-	int cmp;
 	
 	dbg ("Device added: %s\n", udi);
 	
@@ -1681,7 +1829,9 @@ hal_device_added (LibHalContext *ctx __attribute__((__unused__)),
 	
 	qsort (capabilities, n, sizeof (char *), strptrcmp);
 	
-	for (i = 0, j = 0; i < G_N_ELEMENTS (devices) && j < n; i++, cmp = -1) {
+	for (i = 0, j = 0; i < G_N_ELEMENTS (devices) && j < n; i++) {
+		int cmp = -1;
+		
 		while (j < n && (cmp = strcmp (capabilities[j], devices[i].capability)) < 0)
 			j++;
 		
@@ -1704,7 +1854,12 @@ static void
 hal_device_removed (LibHalContext *ctx __attribute__((__unused__)), 
 		    const char *udi)
 {
+	GtkDialog *dialog;
+	
 	dbg ("Device removed: %s\n", udi);
+	
+	if ((dialog = g_hash_table_lookup (dialogs, udi)))
+		gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
 }
 
 /** Invoked when device in the Global Device List acquires a new capability.
@@ -1747,7 +1902,7 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 		       dbus_bool_t is_removed __attribute__((__unused__)), 
 		       dbus_bool_t is_added __attribute__((__unused__)))
 {
-	char *mounting_udi;
+	struct _MountPolicy *policy;
 	gboolean mounted;
 	GSList *l, *n;
 	
@@ -1759,24 +1914,28 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 	if (mounted) {
 		dbg ("Mounted: %s\n", udi);
 		
-		if ((mounting_udi = g_hash_table_lookup (mount_table, udi))) {
+		if ((policy = g_hash_table_lookup (mount_table, udi))) {
 			g_hash_table_remove (mount_table, udi);
-			g_free (mounting_udi);
 			
 			/* add to list of all volumes mounted during lifetime */
 			mounted_volumes = g_slist_append (mounted_volumes, g_strdup (udi));
+			
+			if (policy->apply)
+				gvm_device_mounted (udi);
+			
+			g_free (policy->udi);
+			g_free (policy);
 		} else {
 			dbg ("not in mount queue: %s\n", udi);
+			gvm_device_mounted (udi);
 		}
-		
-		gvm_device_mounted (udi);
 	} else {
 		dbg ("Unmounted: %s\n", udi);
 		
-		/* remove from list of all volumes mounted during lifetime */
+		/* unmount all volumes mounted during lifetime */
 		
 		for (l = mounted_volumes; l != NULL; l = n) {
-			n = g_slist_next (l);
+			n = l->next;
 			if (strcmp (udi, (const char *) l->data) == 0) {
 				g_free (l->data);
 				mounted_volumes = g_slist_delete_link (mounted_volumes, l);
@@ -1795,11 +1954,68 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
  *  @param  message             D-BUS message with parameters
  */
 static void
-hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
-		      const char *udi __attribute__((__unused__)), 
-		      const char *condition_name __attribute__((__unused__)),
-		      const char *condition_detail __attribute__((__unused__)))
+hal_device_condition (LibHalContext *ctx, const char *udi, const char *condition_name,
+		      const char *condition_details __attribute__((__unused__)))
 {
+	if (!strcmp (condition_name, "EjectPressed")) {
+		char *storage, **volumes = NULL;
+		DBusMessage *dmesg, *reply;
+		const char *volume = udi;
+		DBusError error;
+		int i, n;
+		
+		if (!libhal_device_get_property_bool (ctx, udi, "block.is_volume", NULL)) {
+			dbus_error_init (&error);
+			volumes = libhal_find_device_by_capability (ctx, "volume", &n, &error);
+			if (dbus_error_is_set (&error)) {
+				dbus_error_free (&error);
+				return;
+			}
+			
+			volume = NULL;
+			for (i = 0; i < n; i++) {
+				volume = volumes[i];
+				
+				if (!(storage = libhal_device_get_property_string (ctx, volume, "info.parent", NULL)))
+					continue;
+				
+				if (!strcmp (udi, storage)) {
+					libhal_free_string (storage);
+					break;
+				}
+				
+				libhal_free_string (storage);
+			}
+		}
+		
+		dbg ("ejecting %s...\n", volume);
+		
+		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", volume,
+							    "org.freedesktop.Hal.Device.Volume",
+							    "Eject"))) {
+			if (volumes)
+				libhal_free_string_array (volumes);
+			return;
+		}
+		
+		dbus_error_init (&error);
+		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+			dbg ("eject failed for %s: %s\n", volume, error.message);
+			dbus_message_unref (dmesg);
+			dbus_error_free (&error);
+			
+			if (volumes)
+				libhal_free_string_array (volumes);
+			
+			return;
+		}
+		
+		if (volumes)
+			libhal_free_string_array (volumes);
+		
+		dbus_message_unref (dmesg);
+		dbus_message_unref (reply);
+	}
 }
 
 /** Integrate a dbus mainloop. 
@@ -1812,15 +2028,13 @@ hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
 static dbus_bool_t 
 hal_mainloop_integration (LibHalContext *ctx, DBusError *error)
 {
-	DBusConnection *dbus_connection;
-
 	dbus_connection = dbus_bus_get (DBUS_BUS_SYSTEM, error);
-
+	
 	if (dbus_error_is_set (error))
 		return FALSE;
 	
         dbus_connection_setup_with_g_main (dbus_connection, NULL);
-
+	
 	libhal_ctx_set_dbus_connection (ctx, dbus_connection);
 	
 	return TRUE;
@@ -1864,14 +2078,14 @@ gvm_do_hal_init (void)
 		libhal_ctx_free (ctx);
 		return NULL;
 	}
-
+	
 	if (!libhal_ctx_init (ctx, &error)) {
 		warn ("hal_initialize failed: %s", error.message);
 		dbus_error_free (&error);
 		libhal_ctx_free (ctx);
 		return NULL;
 	}
-
+	
 	/*
 	 * Do something to ping the HAL daemon - the above functions will
 	 * succeed even if hald is not running, so long as DBUS is.  But we
@@ -1888,7 +2102,7 @@ gvm_do_hal_init (void)
 	}
 	
 	libhal_free_string_array (devices);
-
+	
 	return ctx;
 }
 
@@ -1957,7 +2171,7 @@ mount_all (LibHalContext *ctx)
 		/* mount the device */
 		if ((dev = libhal_device_get_property_string (ctx, udi, "block.device", &error))) {
 			dbg ("mount_all: mounting %s\n", dev);
-			gvm_device_mount (udi, dev, NULL);
+			gvm_device_mount (udi, FALSE);
 			libhal_free_string (dev);
 		} else {
 			warn ("mount_all: no device for udi=%s: %s", udi, error.message);
@@ -1975,26 +2189,17 @@ mount_all (LibHalContext *ctx)
  *  @param  ctx                 LibHal context
  */
 static void
-unmount_all (LibHalContext *ctx)
+unmount_all (void)
 {
-	char *udi, *device;
-	DBusError error;
 	GSList *l;
 	
-	dbg ("unmounting all volumes that we saw mounted in our life\n");
-	dbus_error_init (&error);
-	for (l = mounted_volumes; l != NULL; l = g_slist_next (l)) {
-		udi = l->data;
+	dbg ("unmounting all volumes that we mounted in our lifetime\n");
+	
+	for (l = mounted_volumes; l != NULL; l = l->next) {
+		const char *udi = l->data;
 		
-		if ((device = libhal_device_get_property_string (ctx, udi, "block.device", &error))) {
-			dbg ("unmount_all: unmounting %s\n", device);
-			gvm_device_unmount (udi, device, NULL);
-			libhal_free_string (device);
-		} else {
-			warn ("unmount_all: no device for udi=%s: %s", udi, error.message);
-			if (dbus_error_is_set (&error))
-				dbus_error_free (&error);
-		}
+		dbg ("unmount_all: unmounting %s...\n", udi);
+		gvm_device_unmount (udi);
 	}
 }
 
@@ -2034,7 +2239,7 @@ sigterm_iochn_data (GIOChannel *source,
 	
 	dbg ("Received SIGTERM, initiating shutdown\n");
 	
-	unmount_all (hal_ctx);
+	unmount_all ();
 	
 	gtk_main_quit ();
 	
@@ -2048,11 +2253,30 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 {
 	dbg ("Received 'die', initiating shutdown\n");
 	
-	unmount_all (hal_ctx);
+	unmount_all ();
 	
 	gtk_main_quit ();
 }
 
+/* assume that display :0 is the system console */
+static int
+gvm_running_on_console (void)
+{
+	const char *str;
+	
+	if (!(str = DisplayString (GDK_DISPLAY ())))
+		return TRUE;
+	
+	if (!(str = strchr (str, ':')))
+		return TRUE;
+	
+	str++;
+	if (str[0] == ':')
+		str++;
+	
+	return strtoul (str, NULL, 10) == 0;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -2066,18 +2290,22 @@ main (int argc, char *argv[])
 	textdomain (PACKAGE);
 	
 	client = gnome_master_client ();
-	if (gvm_get_clipboard ()) {
+	if (gvm_get_clipboard () && gvm_running_on_console ()) {
 		gnome_client_set_restart_style (client, GNOME_RESTART_ANYWAY);
 	} else {
 		gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-		warn ("already running?");
+		
+		if (!gvm_running_on_console ())
+			warn ("not on the system console");
+		else
+			warn ("already running");
+		
 		return 1;
 	}
 	
 	g_signal_connect (client, "die", G_CALLBACK (gvm_die), NULL);
 	
-	hal_ctx = gvm_do_hal_init ();
-	if (!hal_ctx)
+	if (!(hal_ctx = gvm_do_hal_init ()))
 		return 1;
 	
 	gvm_init_config ();
@@ -2098,6 +2326,7 @@ main (int argc, char *argv[])
 	signal (SIGTERM, handle_sigterm);
 	
 	mount_table = g_hash_table_new (g_str_hash, g_str_equal);
+	dialogs = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	mount_all (hal_ctx);
 	
